@@ -300,7 +300,7 @@ void CL_KeepaliveMessage (void)
 		memset(&msg, 0, sizeof(msg));
 		msg.data = buf;
 		msg.maxsize = sizeof(buf);
-		MSG_WriteChar(&msg, svc_nop);
+		MSG_WriteChar(&msg, clc_nop);
 		NetConn_SendUnreliableMessage(cls.netcon, &msg, cls.protocol);
 	}
 }
@@ -639,7 +639,7 @@ static void QW_CL_ParseModelList(void)
 		if (nummodels==MAX_MODELS)
 			Host_Error("Server sent too many model precaches");
 		if (strlen(str) >= MAX_QPATH)
-			Host_Error("Server sent a precache name of %i characters (max %i)", strlen(str), MAX_QPATH - 1);
+			Host_Error("Server sent a precache name of %i characters (max %i)", (int)strlen(str), MAX_QPATH - 1);
 		strlcpy(cl.model_name[nummodels], str, sizeof (cl.model_name[nummodels]));
 	}
 
@@ -673,7 +673,7 @@ static void QW_CL_ParseSoundList(void)
 		if (numsounds==MAX_SOUNDS)
 			Host_Error("Server sent too many sound precaches");
 		if (strlen(str) >= MAX_QPATH)
-			Host_Error("Server sent a precache name of %i characters (max %i)", strlen(str), MAX_QPATH - 1);
+			Host_Error("Server sent a precache name of %i characters (max %i)", (int)strlen(str), MAX_QPATH - 1);
 		strlcpy(cl.sound_name[numsounds], str, sizeof (cl.sound_name[numsounds]));
 	}
 
@@ -996,21 +996,9 @@ void CL_ParseServerInfo (void)
 
 		str = MSG_ReadString();
 		Con_Printf("server gamedir is %s\n", str);
-#if 0
-		// FIXME: change gamedir if needed!
-		if (strcasecmp(gamedirfile, str))
-		{
-			Host_SaveConfig_f();
-			cflag = 1;
-		}
 
-		Com_Gamedir(str); // change gamedir
-
-		if (cflag)
-		{
-			// exec the new config stuff
-		}
-#endif
+		// change gamedir if needed
+		FS_ChangeGameDir(str);
 
 		cl.gametype = GAME_DEATHMATCH;
 		cl.maxclients = 32;
@@ -1091,7 +1079,7 @@ void CL_ParseServerInfo (void)
 			if (nummodels==MAX_MODELS)
 				Host_Error ("Server sent too many model precaches");
 			if (strlen(str) >= MAX_QPATH)
-				Host_Error ("Server sent a precache name of %i characters (max %i)", strlen(str), MAX_QPATH - 1);
+				Host_Error ("Server sent a precache name of %i characters (max %i)", (int)strlen(str), MAX_QPATH - 1);
 			strlcpy (cl.model_name[nummodels], str, sizeof (cl.model_name[nummodels]));
 		}
 		// parse sound precache list
@@ -1103,7 +1091,7 @@ void CL_ParseServerInfo (void)
 			if (numsounds==MAX_SOUNDS)
 				Host_Error("Server sent too many sound precaches");
 			if (strlen(str) >= MAX_QPATH)
-				Host_Error("Server sent a precache name of %i characters (max %i)", strlen(str), MAX_QPATH - 1);
+				Host_Error("Server sent a precache name of %i characters (max %i)", (int)strlen(str), MAX_QPATH - 1);
 			strlcpy (cl.sound_name[numsounds], str, sizeof (cl.sound_name[numsounds]));
 		}
 
