@@ -84,8 +84,9 @@ void Com_BlockFullChecksum (void *buffer, int len, unsigned char *outbuf);
 #endif
 
 // If we still don't know the CPU endianess at this point, we try to guess
+// normally including sys/types.h includes endian.h for the platform, which defines BYTE_ORDER, LITTLE_ENDIAN, and BIG_ENDIAN, however endian.h is a BSD-ism, and may not be present on all platforms (particularly windows)
 #ifndef BYTE_ORDER
-# if defined(WIN32)
+# if defined(WIN32) || defined (__i386) || defined(__amd64)
 #  define BYTE_ORDER LITTLE_ENDIAN
 # else
 #  if defined(SUNOS)
@@ -216,7 +217,7 @@ void COM_Init (void);
 void COM_Shutdown (void);
 void COM_InitGameType (void);
 
-char	*va(const char *format, ...);
+char	*va(const char *format, ...) DP_FUNC_PRINTF(1);
 // does a varargs printf into a temp buffer
 
 
@@ -228,7 +229,7 @@ char	*va(const char *format, ...);
 // return the number of printed characters, excluding the final '\0'
 // or return -1 if the buffer isn't big enough to contain the entire string.
 // buffer is ALWAYS null-terminated
-extern int dpsnprintf (char *buffer, size_t buffersize, const char *format, ...);
+extern int dpsnprintf (char *buffer, size_t buffersize, const char *format, ...) DP_FUNC_PRINTF(3);
 extern int dpvsnprintf (char *buffer, size_t buffersize, const char *format, va_list args);
 
 // A bunch of functions are forbidden for security reasons (and also to please MSVS 2005, for some of them)
