@@ -225,7 +225,6 @@ typedef struct pack_s
 	int ignorecase;  // PK3 ignores case
 	int numfiles;
 	packfile_t *files;
-	struct pack_s *next;
 } pack_t;
 
 
@@ -250,7 +249,6 @@ FUNCTION PROTOTYPES
 void FS_Dir_f(void);
 void FS_Ls_f(void);
 
-static const char *FS_FileExtension (const char *in);
 static searchpath_t *FS_FindFile (const char *name, int* index, qboolean quiet);
 static packfile_t* FS_AddFileToPack (const char* name, pack_t* pack,
 									fs_offset_t offset, fs_offset_t packsize,
@@ -267,8 +265,6 @@ VARIABLES
 
 mempool_t *fs_mempool;
 
-pack_t *packlist = NULL;
-
 searchpath_t *fs_searchpaths = NULL;
 
 #define MAX_FILES_IN_PACK	65536
@@ -276,7 +272,10 @@ searchpath_t *fs_searchpaths = NULL;
 char fs_gamedir[MAX_OSPATH];
 char fs_basedir[MAX_OSPATH];
 
-qboolean fs_modified;   // set true if using non-id files
+// list of active game directories (empty if not running a mod)
+#define MAX_GAMEDIRS 16
+int fs_numgamedirs = 0;
+char fs_gamedirs[MAX_GAMEDIRS][MAX_QPATH];
 
 cvar_t scr_screenshot_name = {0, "scr_screenshot_name","dp", "prefix name for saved screenshots (changes based on -game commandline, as well as which game mode is running)"};
 
@@ -586,8 +585,6 @@ pack_t *FS_LoadPackPK3 (const char *packfile)
 	pack->handle = packhandle;
 	pack->numfiles = eocd.nbentries;
 	pack->files = (packfile_t *)Mem_Alloc(fs_mempool, eocd.nbentries * sizeof(packfile_t));
-	pack->next = packlist;
-	packlist = pack;
 
 	real_nb_files = PK3_BuildFileList (pack, &eocd);
 	if (real_nb_files < 0)
@@ -807,8 +804,6 @@ pack_t *FS_LoadPackPAK (const char *packfile)
 	pack->handle = packhandle;
 	pack->numfiles = 0;
 	pack->files = (packfile_t *)Mem_Alloc(fs_mempool, numpackfiles * sizeof(packfile_t));
-	pack->next = packlist;
-	packlist = pack;
 
 	// parse the directory
 	for (i = 0;i < numpackfiles;i++)
@@ -1032,7 +1027,7 @@ void FS_AddGameHierarchy (const char *dir)
 FS_FileExtension
 ============
 */
-static const char *FS_FileExtension (const char *in)
+const char *FS_FileExtension (const char *in)
 {
 	const char *separator, *backslash, *colon, *dot;
 
@@ -1052,6 +1047,190 @@ static const char *FS_FileExtension (const char *in)
 }
 
 
+/*
+================
+FS_ClearSearchPath
+================
+*/
+void FS_ClearSearchPath (void)
+{
+	while (fs_searchpaths)
+	{
+		searchpath_t *search = fs_searchpaths;
+		fs_searchpaths = search->next;
+		if (search->pack)
+		{
+			if (search->pack->files)
+				Mem_Free(search->pack->files);
+			Mem_Free(search->pack);
+		}
+		Mem_Free(search);
+	}
+}
+
+
+/*
+================
+FS_Rescan_f
+================
+*/
+void FS_Rescan_f (void)
+{
+	int i;
+	qboolean fs_modified = false;
+
+	FS_ClearSearchPath();
+
+	// add the game-specific paths
+	// gamedirname1 (typically id1)
+	FS_AddGameHierarchy (gamedirname1);
+	// update the com_modname (used for server info)
+	strlcpy(com_modname, gamedirname1, sizeof(com_modname));
+
+	// add the game-specific path, if any
+	// (only used for mission packs and the like, which should set fs_modified)
+	if (gamedirname2)
+	{
+		fs_modified = true;
+		FS_AddGameHierarchy (gamedirname2);
+	}
+
+	// -game <gamedir>
+	// Adds basedir/gamedir as an override game
+	// LordHavoc: now supports multiple -game directories
+	// set the com_modname (reported in server info)
+	for (i = 0;i < fs_numgamedirs;i++)
+	{
+		fs_modified = true;
+		FS_AddGameHierarchy (fs_gamedirs[i]);
+		// update the com_modname (used server info)
+		strlcpy (com_modname, fs_gamedirs[i], sizeof (com_modname));
+	}
+
+	// set the default screenshot name to either the mod name or the
+	// gamemode screenshot name
+	if (strcmp(com_modname, gamedirname1))
+		Cvar_SetQuick (&scr_screenshot_name, com_modname);
+	else
+		Cvar_SetQuick (&scr_screenshot_name, gamescreenshotname);
+
+	// If "-condebug" is in the command line, remove the previous log file
+	if (COM_CheckParm ("-condebug") != 0)
+		unlink (va("%s/qconsole.log", fs_gamedir));
+
+	// look for the pop.lmp file and set registered to true if it is found
+	if ((gamemode == GAME_NORMAL || gamemode == GAME_HIPNOTIC || gamemode == GAME_ROGUE) && !FS_FileExists("gfx/pop.lmp"))
+	{
+		if (fs_modified)
+			Con_Print("Playing shareware version, with modification.\nwarning: most mods require full quake data.\n");
+		else
+			Con_Print("Playing shareware version.\n");
+	}
+	else
+	{
+		Cvar_Set ("registered", "1");
+		if (gamemode == GAME_NORMAL || gamemode == GAME_HIPNOTIC || gamemode == GAME_ROGUE)
+			Con_Print("Playing registered version.\n");
+	}
+}
+
+
+/*
+================
+FS_ChangeGameDir
+================
+*/
+void Host_SaveConfig_f (void);
+void Host_LoadConfig_f (void);
+qboolean FS_ChangeGameDir(const char *string)
+{
+	// if already using the requested gamedir, do nothing
+	if (fs_numgamedirs == 1 && !strcmp(fs_gamedirs[0], string))
+		return false;
+
+	// if string is nasty, reject it
+	if(FS_CheckNastyPath(string, true)) // overflowed or nasty?
+	{
+		Con_Printf("FS_ChangeGameDir(\"%s\"): nasty filename rejected\n", string);
+		return false;
+	}
+
+	// save the current config
+	Host_SaveConfig_f();
+
+	// change to the new gamedir
+	fs_numgamedirs = 1;
+	strlcpy(fs_gamedirs[0], string, sizeof(fs_gamedirs[0]));
+
+	// reinitialize filesystem to detect the new paks
+	FS_Rescan_f();
+
+	// exec the new config
+	Host_LoadConfig_f();
+
+	// reinitialize the loaded sounds
+	S_Reload_f();
+
+	// reinitialize renderer (this reloads hud/console background/etc)
+	R_Modules_Restart();
+
+	return true;
+}
+
+/*
+================
+FS_GameDir_f
+================
+*/
+void FS_GameDir_f (void)
+{
+	int i;
+
+	if (Cmd_Argc() < 2)
+	{
+		Con_Printf("gamedirs active:");
+		for (i = 0;i < fs_numgamedirs;i++)
+			Con_Printf(" %s", fs_gamedirs[i]);
+		Con_Printf("\n");
+		return;
+	}
+
+	if (cls.state != ca_disconnected || sv.active)
+	{
+		Con_Printf("Can not change gamedir while client is connected or server is running!\n");
+		return;
+	}
+
+	Host_SaveConfig_f();
+
+	fs_numgamedirs = 0;
+	for (i = 1;i < Cmd_Argc() && fs_numgamedirs < MAX_GAMEDIRS;i++)
+	{
+		// if string is nasty, reject it
+		if(FS_CheckNastyPath(Cmd_Argv(i), true)) // overflowed or nasty?
+		{
+			Con_Printf("FS_GameDir_f(\"%s\"): nasty filename rejected\n", Cmd_Argv(i));
+			continue;
+		}
+
+		strlcpy(fs_gamedirs[fs_numgamedirs], Cmd_Argv(i), sizeof(fs_gamedirs[fs_numgamedirs]));
+		fs_numgamedirs++;
+	}
+
+	// reinitialize filesystem to detect the new paks
+	FS_Rescan_f();
+
+	// exec the new config
+	Host_LoadConfig_f();
+
+	// reinitialize the loaded sounds
+	S_Reload_f();
+
+	// reinitialize renderer (this reloads hud/console background/etc)
+	R_Modules_Restart();
+}
+
+
 /*
 ================
 FS_Init
@@ -1060,7 +1239,6 @@ FS_Init
 void FS_Init (void)
 {
 	int i;
-	searchpath_t *search;
 
 	fs_mempool = Mem_AllocPool("file management", 0, NULL);
 
@@ -1105,92 +1283,32 @@ void FS_Init (void)
 	if (fs_basedir[0] && fs_basedir[strlen(fs_basedir) - 1] != '/' && fs_basedir[strlen(fs_basedir) - 1] != '\\')
 		strlcat(fs_basedir, "/", sizeof(fs_basedir));
 
-	// -path <dir or packfile> [<dir or packfile>] ...
-	// Fully specifies the exact search path, overriding the generated one
-// COMMANDLINEOPTION: Filesystem: -path <path ..> specifies the full search path manually, overriding the generated one, example: -path c:\quake\id1 c:\quake\pak0.pak c:\quake\pak1.pak (not recommended)
-	i = COM_CheckParm ("-path");
-	if (i)
+	// -game <gamedir>
+	// Adds basedir/gamedir as an override game
+	// LordHavoc: now supports multiple -game directories
+	for (i = 1;i < com_argc && fs_numgamedirs < MAX_GAMEDIRS;i++)
 	{
-		fs_modified = true;
-		while (++i < com_argc)
+		if (!com_argv[i])
+			continue;
+		if (!strcmp (com_argv[i], "-game") && i < com_argc-1)
 		{
-			if (!com_argv[i] || com_argv[i][0] == '+' || com_argv[i][0] == '-')
-				break;
-
-			if(!FS_AddPack_Fullpath(com_argv[i], NULL, false))
-			{
-				search = (searchpath_t *)Mem_Alloc(fs_mempool, sizeof(searchpath_t));
-				strlcpy (search->filename, com_argv[i], sizeof (search->filename));
-				search->next = fs_searchpaths;
-				fs_searchpaths = search;
-			}
+			i++;
+			// add the gamedir to the list of active gamedirs
+			strlcpy (fs_gamedirs[fs_numgamedirs], com_argv[i], sizeof(fs_gamedirs[fs_numgamedirs]));
+			fs_numgamedirs++;
 		}
 	}
-	else
-	{
-		// add the game-specific paths
-		// gamedirname1 (typically id1)
-		FS_AddGameHierarchy (gamedirname1);
 
-		// add the game-specific path, if any
-		if (gamedirname2)
-		{
-			fs_modified = true;
-			FS_AddGameHierarchy (gamedirname2);
-		}
-
-		// set the com_modname (reported in server info)
-		strlcpy(com_modname, gamedirname1, sizeof(com_modname));
-
-		// -game <gamedir>
-		// Adds basedir/gamedir as an override game
-		// LordHavoc: now supports multiple -game directories
-		for (i = 1;i < com_argc;i++)
-		{
-			if (!com_argv[i])
-				continue;
-			if (!strcmp (com_argv[i], "-game") && i < com_argc-1)
-			{
-				i++;
-				fs_modified = true;
-				FS_AddGameHierarchy (com_argv[i]);
-				// update the com_modname
-				strlcpy (com_modname, com_argv[i], sizeof (com_modname));
-			}
-		}
-
-		// If "-condebug" is in the command line, remove the previous log file
-		if (COM_CheckParm ("-condebug") != 0)
-			unlink (va("%s/qconsole.log", fs_gamedir));
-	}
-
-	// look for the pop.lmp file and set registered to true if it is found
-	if (gamemode == GAME_NORMAL && !FS_FileExists("gfx/pop.lmp"))
-	{
-		if (fs_modified)
-			Con_Print("Playing shareware version, with modification.\nwarning: most mods require full quake data.\n");
-		else
-			Con_Print("Playing shareware version.\n");
-	}
-	else
-	{
-		Cvar_Set ("registered", "1");
-		if (gamemode == GAME_NORMAL || gamemode == GAME_HIPNOTIC || gamemode == GAME_ROGUE)
-			Con_Print("Playing registered version.\n");
-	}
-
-	// set the default screenshot name to either the mod name or the
-	// gamemode screenshot name
-	if (fs_modified)
-		Cvar_SetQuick (&scr_screenshot_name, com_modname);
-	else
-		Cvar_SetQuick (&scr_screenshot_name, gamescreenshotname);
+	// update the searchpath
+	FS_Rescan_f();
 }
 
 void FS_Init_Commands(void)
 {
 	Cvar_RegisterVariable (&scr_screenshot_name);
 
+	Cmd_AddCommand ("gamedir", FS_GameDir_f, "changes active gamedir list (can take multiple arguments), not including base directory (example usage: gamedir ctf)");
+	Cmd_AddCommand ("fs_rescan", FS_Rescan_f, "rescans filesystem for new pack archives and any other changes");
 	Cmd_AddCommand ("path", FS_Path_f, "print searchpath (game directories and archives)");
 	Cmd_AddCommand ("dir", FS_Dir_f, "list files in searchpath matching an * filename pattern, one per line");
 	Cmd_AddCommand ("ls", FS_Ls_f, "list files in searchpath matching an * filename pattern, multiple per line");
@@ -1314,7 +1432,7 @@ qfile_t *FS_OpenPackedFile (pack_t* pack, int pack_ind)
 	if (lseek (pack->handle, pfile->offset, SEEK_SET) == -1)
 	{
 		Con_Printf ("FS_OpenPackedFile: can't lseek to %s in %s (offset: %d)\n",
-					pfile->name, pack->filename, pfile->offset);
+					pfile->name, pack->filename, (int) pfile->offset);
 		return NULL;
 	}
 
@@ -1385,8 +1503,12 @@ Return true if the path should be rejected due to one of the following:
    or are just not a good idea for a mod to be using.
 ====================
 */
-int FS_CheckNastyPath (const char *path)
+int FS_CheckNastyPath (const char *path, qboolean isgamedir)
 {
+	// all: never allow an empty path, as for gamedir it would access the parent directory and a non-gamedir path it is just useless
+	if (!path[0])
+		return 2;
+
 	// Windows: don't allow \ in filenames (windows-only), period.
 	// (on Windows \ is a directory separator, but / is also supported)
 	if (strstr(path, "\\"))
@@ -1403,14 +1525,34 @@ int FS_CheckNastyPath (const char *path)
 	if (strstr(path, "//"))
 		return 1; // non-portable attempt to go to parent directory
 
-	// all: don't allow going to current directory (./) or parent directory (../ or /../)
-	if (strstr(path, "./"))
+	// all: don't allow going to parent directory (../ or /../)
+	if (strstr(path, ".."))
 		return 2; // attempt to go outside the game directory
 
 	// Windows and UNIXes: don't allow absolute paths
 	if (path[0] == '/')
 		return 2; // attempt to go outside the game directory
 
+	// all: don't allow . characters before the last slash (it should only be used in filenames, not path elements), this catches all imaginable cases of ./, ../, .../, etc
+	if (strchr(path, '.'))
+	{
+		if (isgamedir)
+		{
+			// gamedir is entirely path elements, so simply forbid . entirely
+			return 2;
+		}
+		if (strchr(path, '.') < strrchr(path, '/'))
+			return 2; // possible attempt to go outside the game directory
+	}
+
+	// all: forbid trailing slash on gamedir
+	if (isgamedir && path[strlen(path)-1] == '/')
+		return 2;
+
+	// all: forbid leading dot on any filename for any reason
+	if (strstr(path, "/."))
+		return 2; // attempt to go outside the game directory
+
 	// after all these checks we're pretty sure it's a / separated filename
 	// and won't do much if any harm
 	return false;
@@ -1546,7 +1688,7 @@ Open a file. The syntax is the same as fopen
 */
 qfile_t* FS_Open (const char* filepath, const char* mode, qboolean quiet, qboolean nonblocking)
 {
-	if (FS_CheckNastyPath(filepath))
+	if (FS_CheckNastyPath(filepath, false))
 	{
 		Con_Printf("FS_Open(\"%s\", \"%s\", %s): nasty filename rejected\n", filepath, mode, quiet ? "true" : "false");
 		return NULL;
@@ -2006,6 +2148,19 @@ fs_offset_t FS_Tell (qfile_t* file)
 }
 
 
+/*
+====================
+FS_FileSize
+
+Give the total size of a file
+====================
+*/
+fs_offset_t FS_FileSize (qfile_t* file)
+{
+	return file->real_length;
+}
+
+
 /*
 ====================
 FS_Purge
