@@ -134,7 +134,9 @@ int LHNETADDRESS_FromString(lhnetaddress_t *address, const char *string, int def
 		return 1;
 	}
 	// try to parse as dotted decimal ipv4 address first
-	if (sscanf(name, "%d.%d.%d.%d", &d1, &d2, &d3, &d4) == 4 && (unsigned int)d1 < 256 && (unsigned int)d2 < 256 && (unsigned int)d3 < 256 && (unsigned int)d4 < 256)
+	// note this supports partial ip addresses
+	d1 = d2 = d3 = d4 = 0;
+	if (sscanf(name, "%d.%d.%d.%d", &d1, &d2, &d3, &d4) >= 1 && (unsigned int)d1 < 256 && (unsigned int)d2 < 256 && (unsigned int)d3 < 256 && (unsigned int)d4 < 256)
 	{
 		// parsed a valid ipv4 address
 		address->addresstype = LHNETADDRESSTYPE_INET4;
@@ -375,6 +377,130 @@ int LHNETADDRESS_Compare(const lhnetaddress_t *address1, const lhnetaddress_t *a
 	}
 }
 
+static int LHNET_memmaskcmp(const void *buf1, const void *buf2, const void *mask, size_t len)
+{
+	const unsigned char *a = (const unsigned char *) buf1;
+	const unsigned char *b = (const unsigned char *) buf2;
+	const unsigned char *m = (const unsigned char *) mask;
+	for(; len; ++a, ++b, ++m, --len)
+		if((*a ^ *b) & *m)
+			return 1;
+	return 0;
+}
+
+int LHNETADDRESS_NetworkFromString(lhnetaddress_t *address, lhnetaddress_t *netmask, const char *string)
+{
+	char buf[128];
+	const char *slash = strchr(string, '/');
+	int i;
+	if(slash)
+	{
+		long bits;
+		char *end = NULL;
+
+		if((size_t)(slash - string) >= sizeof(buf))
+			return 0; // too long
+		memcpy(buf, string, slash - string);
+		buf[slash - string] = 0;
+		if(!LHNETADDRESS_FromString(address, buf, 0))
+			return 0;
+
+		bits = strtol(slash + 1, &end, 0);
+		if(!slash[1] || (end && *end)) // error in input
+		{
+			if(!LHNETADDRESS_FromString(netmask, slash + 1, 0))
+				return 0;
+			return 1;
+		}
+		else
+		{
+			memcpy(netmask, address, sizeof(*netmask));
+			// fill the first bits bits with ones, the rest with zeros
+			switch(address->addresstype)
+			{
+				case LHNETADDRESSTYPE_LOOP:
+					return 0; // does not make sense
+				case LHNETADDRESSTYPE_INET4:
+					if(bits > 32)
+						bits = 32;
+					for(i = 0; i < bits / 8; ++i)
+						netmask->addressdata.inet4.address[i] = 0xFF;
+					if(bits % 8)
+						netmask->addressdata.inet4.address[bits / 8] = 0xFF & (0xFF << (8 - bits % 8));
+					return 1;
+				case LHNETADDRESSTYPE_INET6:
+					if(bits > 128)
+						bits = 128;
+					for(i = 0; i < bits / 16; ++i)
+						netmask->addressdata.inet6.address[i] = 0xFFFF;
+					if(bits % 16)
+						netmask->addressdata.inet6.address[bits / 16] = 0xFFFF & (0xFFFF << (16 - bits % 16));
+					return 1;
+				default:
+					return 0;
+			}
+
+		}
+	}
+	else
+	{
+		// load the address
+		if(!LHNETADDRESS_FromString(address, string, 0))
+			return 0;
+		// fill netmask with the all-ones address
+		memcpy(netmask, address, sizeof(*netmask));
+		switch(address->addresstype)
+		{
+			case LHNETADDRESSTYPE_LOOP:
+				return 1;
+			case LHNETADDRESSTYPE_INET4:
+				memset(netmask->addressdata.inet4.address, 0xFF, sizeof(netmask->addressdata.inet4.address));
+				return 1;
+			case LHNETADDRESSTYPE_INET6:
+				memset(netmask->addressdata.inet6.address, 0xFF, sizeof(netmask->addressdata.inet6.address));
+				return 1;
+			default:
+				return 0;
+		}
+	}
+	return 0;
+}
+
+int LHNETADDRESS_InNetwork(const lhnetaddress_t *peer, const lhnetaddress_t *address, const lhnetaddress_t *netmask)
+{
+	if(!peer || !address)
+		return 0;
+	if(!netmask)
+		return LHNETADDRESS_Compare(peer, address) == 0;
+	if(peer->addresstype != address->addresstype)
+		return 0;
+	if(peer->addresstype != netmask->addresstype)
+		return 0;
+	switch(peer->addresstype)
+	{
+	case LHNETADDRESSTYPE_LOOP:
+		return 1; // loopback is always in the same "network"
+	case LHNETADDRESSTYPE_INET4:
+		if(peer->addressdata.inet4.family != address->addressdata.inet4.family)
+			return 0;
+		if(peer->addressdata.inet4.family != netmask->addressdata.inet4.family)
+			return 0;
+		if(LHNET_memmaskcmp(peer->addressdata.inet4.address, address->addressdata.inet4.address, netmask->addressdata.inet4.address, sizeof(peer->addressdata.inet4.address)))
+			return 0;
+		return 1;
+	case LHNETADDRESSTYPE_INET6:
+		if(peer->addressdata.inet6.family != address->addressdata.inet6.family)
+			return 0;
+		if(peer->addressdata.inet6.family != netmask->addressdata.inet6.family)
+			return 0;
+		if(LHNET_memmaskcmp(peer->addressdata.inet6.address, address->addressdata.inet6.address, netmask->addressdata.inet6.address, sizeof(peer->addressdata.inet6.address)))
+			return 0;
+		return 1;
+	default:
+		return 0;
+	}
+}
+
 typedef struct lhnetpacket_s
 {
 	void *data;
