@@ -27,7 +27,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #define DPMASTER_PORT 27950
 
 // note this defaults on for dedicated servers, off for listen servers
-cvar_t sv_public = {0, "sv_public", "0", "advertises this server on the master server (so that players can find it in the server browser)"};
+cvar_t sv_public = {0, "sv_public", "0", "1: advertises this server on the master server (so that players can find it in the server browser); 0: allow direct queries only; -1: do not respond to direct queries; -2: do not allow anyone to connect"};
 static cvar_t sv_heartbeatperiod = {CVAR_SAVE, "sv_heartbeatperiod", "120", "how often to send heartbeat in seconds (only used if sv_public is 1)"};
 
 static cvar_t sv_masters [] =
@@ -70,6 +70,7 @@ static double nextheartbeattime = 0;
 sizebuf_t net_message;
 static unsigned char net_message_buf[NET_MAXMESSAGE];
 
+cvar_t net_banlist = {CVAR_SAVE, "net_banlist","", "space separated list of networks/IPs to be banned in a.b.c.d/nn or a.b.c.d/e.f.g.h notation"};
 cvar_t net_messagetimeout = {0, "net_messagetimeout","300", "drops players who have not sent any packets for this many seconds"};
 cvar_t net_connecttimeout = {0, "net_connecttimeout","10", "after requesting a connection, the client must reply within this many seconds or be dropped (cuts down on connect floods)"};
 cvar_t net_connectfloodblockingtimeout = {0, "net_connectfloodblockingtimeout", "5", "when a connection packet is received, it will block all future connect packets from that IP address for this many seconds (cuts down on connect floods)"};
@@ -126,6 +127,10 @@ cvar_t sv_netport = {0, "port", "26000", "server port for players to connect to"
 cvar_t net_address = {0, "net_address", "0.0.0.0", "network address to open ports on"};
 //cvar_t net_netaddress_ipv6 = {0, "net_address_ipv6", "[0:0:0:0:0:0:0:0]", "network address to open ipv6 ports on"};
 
+char net_extresponse[NET_EXTRESPONSE_MAX][1400];
+int net_extresponse_count = 0;
+int net_extresponse_last = 0;
+
 // ServerList interface
 serverlist_mask_t serverlist_andmasks[SERVERLIST_ANDMASKCOUNT];
 serverlist_mask_t serverlist_ormasks[SERVERLIST_ORMASKCOUNT];
@@ -576,7 +581,7 @@ int NetConn_SendUnreliableMessage(netconn_t *conn, sizebuf_t *data, protocolvers
 		{
 			if (conn->message.cursize > (int)sizeof(conn->sendMessage))
 			{
-				Con_Printf("NetConn_SendUnreliableMessage: reliable message too big (%u > %u)\n", conn->message.cursize, sizeof(conn->sendMessage));
+				Con_Printf("NetConn_SendUnreliableMessage: reliable message too big (%u > %u)\n", conn->message.cursize, (int)sizeof(conn->sendMessage));
 				conn->message.overflowed = true;
 				return -1;
 			}
@@ -1092,14 +1097,14 @@ void NetConn_ConnectionEstablished(lhnetsocket_t *mysocket, lhnetaddress_t *peer
 		Cmd_ForwardStringToServer("new");
 	if (cls.protocol == PROTOCOL_QUAKE)
 	{
-		// write a keepalive (svc_nop) as it seems to greatly improve the
+		// write a keepalive (clc_nop) as it seems to greatly improve the
 		// chances of connecting to a netquake server
 		sizebuf_t msg;
 		unsigned char buf[4];
 		memset(&msg, 0, sizeof(msg));
 		msg.data = buf;
 		msg.maxsize = sizeof(buf);
-		MSG_WriteChar(&msg, svc_nop);
+		MSG_WriteChar(&msg, clc_nop);
 		NetConn_SendUnreliableMessage(cls.netcon, &msg, cls.protocol);
 	}
 }
@@ -1166,6 +1171,66 @@ static void NetConn_ClientParsePacket_ServerList_UpdateCache(int n)
 	serverlist_cache[n].query = SQS_QUERIED;
 }
 
+static int NetConn_IsBanned(lhnetaddress_t *peeraddress)
+{
+	const char *banlist = net_banlist.string;
+	const char *p;
+	char buf[128];
+	size_t len;
+	lhnetaddress_t address, netmask;
+
+	if(!*banlist)
+		return false; // no ban list -> no parsing
+
+	if(peeraddress->addresstype == LHNETADDRESSTYPE_LOOP)
+		return false; // can't ban local
+
+	if(peeraddress->addresstype == LHNETADDRESSTYPE_NONE)
+		return false; // WTF?
+
+	while(*banlist)
+	{
+		while(*banlist == ' ')
+			++banlist;
+		if(!*banlist)
+			break;
+		p = strchr(banlist, ' ');
+		if(!p)
+			p = banlist + strlen(banlist);
+
+		len = p - banlist;
+		if(len < sizeof(buf))
+		{
+			memcpy(buf, banlist, len);
+			buf[len] = 0;
+
+			if(LHNETADDRESS_NetworkFromString(&address, &netmask, buf))
+			{
+				if(developer.integer >= 100)
+				{
+					char addressstring[128];
+					LHNETADDRESS_ToString(&address, addressstring, sizeof(addressstring), false);
+					Con_DPrintf("Banlist check: address %s/", addressstring);
+					LHNETADDRESS_ToString(&netmask, addressstring, sizeof(addressstring), false);
+					Con_DPrintf("%s against ", addressstring);
+					LHNETADDRESS_ToString(peeraddress, addressstring, sizeof(addressstring), false);
+					Con_DPrintf("%s\n", addressstring);
+				}
+				if(LHNETADDRESS_InNetwork(peeraddress, &address, &netmask))
+					return true;
+			}
+			else
+				Con_DPrintf("invalid address in net_banlist: %s\n", buf);
+		}
+		else
+			Con_DPrintf("invalid address in net_banlist: (too long)\n");
+
+		banlist = p;
+	}
+
+	return false;
+}
+
 static int NetConn_ClientParsePacket(lhnetsocket_t *mysocket, unsigned char *data, int length, lhnetaddress_t *peeraddress)
 {
 	qboolean fromserver;
@@ -1233,6 +1298,9 @@ static int NetConn_ClientParsePacket(lhnetsocket_t *mysocket, unsigned char *dat
 			serverlist_info_t *info;
 			int n;
 
+			if(NetConn_IsBanned(peeraddress))
+				return true; // just ignore
+
 			string += 13;
 			// search the cache for this server and update it
 			n = NetConn_ClientParsePacket_ServerList_ProcessReply(addressstring2);
@@ -1343,6 +1411,15 @@ static int NetConn_ClientParsePacket(lhnetsocket_t *mysocket, unsigned char *dat
 			serverlist_querywaittime = realtime + 3;
 			return true;
 		}
+		if (!strncmp(string, "extResponse ", 12))
+		{
+			++net_extresponse_count;
+			if(net_extresponse_count > NET_EXTRESPONSE_MAX)
+				net_extresponse_count = NET_EXTRESPONSE_MAX;
+			net_extresponse_last = (net_extresponse_last + 1) % NET_EXTRESPONSE_MAX;
+			dpsnprintf(net_extresponse[net_extresponse_last], sizeof(net_extresponse[net_extresponse_last]), "'%s' %s", addressstring2, string + 12);
+			return true;
+		}
 		if (!strncmp(string, "ping", 4))
 		{
 			if (developer.integer >= 10)
@@ -1838,7 +1915,7 @@ static int NetConn_ServerParsePacket(lhnetsocket_t *mysocket, unsigned char *dat
 			Com_HexDumpToConsole(data, length);
 		}
 
-		if (length >= 12 && !memcmp(string, "getchallenge", 12))
+		if (length >= 12 && !memcmp(string, "getchallenge", 12) && sv_public.integer > -2)
 		{
 			for (i = 0, best = 0, besttime = realtime;i < MAX_CHALLENGES;i++)
 			{
@@ -1860,7 +1937,7 @@ static int NetConn_ServerParsePacket(lhnetsocket_t *mysocket, unsigned char *dat
 			NetConn_WriteString(mysocket, va("\377\377\377\377challenge %s", challenge[i].string), peeraddress);
 			return true;
 		}
-		if (length > 8 && !memcmp(string, "connect\\", 8))
+		if (length > 8 && !memcmp(string, "connect\\", 8) && sv_public.integer > -2)
 		{
 			string += 7;
 			length -= 7;
@@ -1875,6 +1952,13 @@ static int NetConn_ServerParsePacket(lhnetsocket_t *mysocket, unsigned char *dat
 			if (i == MAX_CHALLENGES)
 				return true;
 
+			if(NetConn_IsBanned(peeraddress))
+			{
+				Con_Printf("Rejecting connection from banned client %s.\n", addressstring2);
+				NetConn_WriteString(mysocket, "\377\377\377\377reject You are not welcome on this server.", peeraddress);
+				return true;
+			}
+
 			// check engine protocol
 			if (strcmp(SearchInfostring(string, "protocol"), "darkplaces 3"))
 			{
@@ -1943,7 +2027,7 @@ static int NetConn_ServerParsePacket(lhnetsocket_t *mysocket, unsigned char *dat
 
 			return true;
 		}
-		if (length >= 7 && !memcmp(string, "getinfo", 7))
+		if (length >= 7 && !memcmp(string, "getinfo", 7) && sv_public.integer > -1)
 		{
 			const char *challenge = NULL;
 
@@ -1959,7 +2043,7 @@ static int NetConn_ServerParsePacket(lhnetsocket_t *mysocket, unsigned char *dat
 			}
 			return true;
 		}
-		if (length >= 9 && !memcmp(string, "getstatus", 9))
+		if (length >= 9 && !memcmp(string, "getstatus", 9) && sv_public.integer > -1)
 		{
 			const char *challenge = NULL;
 
@@ -2045,6 +2129,8 @@ static int NetConn_ServerParsePacket(lhnetsocket_t *mysocket, unsigned char *dat
 				Con_Printf("Datagram_ParseConnectionless: received CCREQ_CONNECT from %s.\n", addressstring2);
 			if (length < (int)strlen("QUAKE") + 1 + 1)
 				break;
+			if(sv_public.integer <= -2)
+				break;
 
 			if (memcmp(data, "QUAKE", strlen("QUAKE") + 1) != 0 || (int)data[strlen("QUAKE") + 1] != NET_PROTOCOL_VERSION)
 			{
@@ -2142,6 +2228,8 @@ static int NetConn_ServerParsePacket(lhnetsocket_t *mysocket, unsigned char *dat
 		case CCREQ_SERVER_INFO:
 			if (developer.integer >= 10)
 				Con_Printf("Datagram_ParseConnectionless: received CCREQ_SERVER_INFO from %s.\n", addressstring2);
+			if(sv_public.integer <= -1)
+				break;
 			if (sv.active && !strcmp(MSG_ReadString(), "QUAKE"))
 			{
 				int numclients;
@@ -2171,6 +2259,8 @@ static int NetConn_ServerParsePacket(lhnetsocket_t *mysocket, unsigned char *dat
 		case CCREQ_PLAYER_INFO:
 			if (developer.integer >= 10)
 				Con_Printf("Datagram_ParseConnectionless: received CCREQ_PLAYER_INFO from %s.\n", addressstring2);
+			if(sv_public.integer <= -1)
+				break;
 			if (sv.active)
 			{
 				int playerNumber, activeNumber, clientNumber;
@@ -2202,6 +2292,8 @@ static int NetConn_ServerParsePacket(lhnetsocket_t *mysocket, unsigned char *dat
 		case CCREQ_RULE_INFO:
 			if (developer.integer >= 10)
 				Con_Printf("Datagram_ParseConnectionless: received CCREQ_RULE_INFO from %s.\n", addressstring2);
+			if(sv_public.integer <= -1)
+				break;
 			if (sv.active)
 			{
 				char *prevCvarName;
@@ -2379,7 +2471,7 @@ void NetConn_Heartbeat(int priority)
 
 	// make advertising optional and don't advertise singleplayer games, and
 	// only send a heartbeat as often as the admin wants
-	if (sv.active && sv_public.integer && svs.maxclients >= 2 && (priority > 1 || realtime > nextheartbeattime))
+	if (sv.active && sv_public.integer > 0 && svs.maxclients >= 2 && (priority > 1 || realtime > nextheartbeattime))
 	{
 		nextheartbeattime = realtime + sv_heartbeatperiod.value;
 		for (masternum = 0;sv_masters[masternum].name;masternum++)
@@ -2449,6 +2541,25 @@ void Net_SlistQW_f(void)
 		ServerList_QueryList(false, true);
 }
 
+void Net_Banlist_Test_f(void)
+{
+	int i;
+	for(i = 1; i < Cmd_Argc(); ++i)
+	{
+		lhnetaddress_t addr;
+		const char *addrstr = Cmd_Argv(i);
+		if(LHNETADDRESS_FromString(&addr, addrstr, 0))
+		{
+			if(NetConn_IsBanned(&addr))
+				Con_Printf("%s is banned\n", addrstr);
+			else
+				Con_Printf("%s is free to join\n", addrstr);
+		}
+		else
+			Con_Printf("Invalid address %s\n", addrstr);
+	}
+}
+
 void NetConn_Init(void)
 {
 	int i;
@@ -2458,6 +2569,8 @@ void NetConn_Init(void)
 	Cmd_AddCommand("net_slist", Net_Slist_f, "query dp master servers and print all server information");
 	Cmd_AddCommand("net_slistqw", Net_SlistQW_f, "query qw master servers and print all server information");
 	Cmd_AddCommand("heartbeat", Net_Heartbeat_f, "send a heartbeat to the master server (updates your server information)");
+	Cmd_AddCommand("net_banlist_test", Net_Banlist_Test_f, "tests if a given IP is banned");
+	Cvar_RegisterVariable(&net_banlist);
 	Cvar_RegisterVariable(&net_slist_queriespersecond);
 	Cvar_RegisterVariable(&net_slist_queriesperframe);
 	Cvar_RegisterVariable(&net_slist_timeout);
@@ -2483,7 +2596,7 @@ void NetConn_Init(void)
 	{
 		if (LHNETADDRESS_FromString(&tempaddress, com_argv[i + 1], 0) == 1)
 		{
-			Con_Printf("-ip option used, setting net_address to \"%s\"\n");
+			Con_Printf("-ip option used, setting net_address to \"%s\"\n", com_argv[i + 1]);
 			Cvar_SetQuick(&net_address, com_argv[i + 1]);
 		}
 		else
