@@ -336,9 +336,9 @@ void SV_AddUpdates (void);
 void SV_ClientThink (void);
 
 void SV_ClientPrint(const char *msg);
-void SV_ClientPrintf(const char *fmt, ...);
+void SV_ClientPrintf(const char *fmt, ...) DP_FUNC_PRINTF(1);
 void SV_BroadcastPrint(const char *msg);
-void SV_BroadcastPrintf(const char *fmt, ...);
+void SV_BroadcastPrintf(const char *fmt, ...) DP_FUNC_PRINTF(1);
 
 void SV_Physics (void);
 void SV_Physics_ClientEntity (prvm_edict_t *ent);
