@@ -724,6 +724,7 @@ void S_Init(void)
 	Cmd_AddCommand("soundlist", S_SoundList_f, "list loaded sounds");
 	Cmd_AddCommand("soundinfo", S_SoundInfo_f, "print sound system information (such as channels and speed)");
 	Cmd_AddCommand("snd_restart", S_Restart_f, "restart sound system");
+	Cmd_AddCommand("snd_reload", S_Reload_f, "reload all sound files");
 
 	Cvar_RegisterVariable(&nosound);
 	Cvar_RegisterVariable(&snd_precache);
@@ -770,6 +771,28 @@ void S_Terminate (void)
 }
 
 
+/*
+==================
+S_Reload_f
+==================
+*/
+void S_Reload_f (void)
+{
+	int i;
+
+	// stop any active sounds
+	S_StopAllSounds();
+
+	// because the ambient sounds will be freed, clear the pointers
+	for (i = 0;i < (int)sizeof (ambient_sfxs) / (int)sizeof (ambient_sfxs[0]);i++)
+		ambient_sfxs[i] = NULL;
+
+	// now free all sounds
+	while (known_sfx != NULL)
+		S_FreeSfx (known_sfx, true);
+}
+
+
 /*
 ==================
 S_FindName
@@ -789,6 +812,7 @@ sfx_t *S_FindName (const char *name)
 	}
 
 	// Look for this sound in the list of known sfx
+	// TODO: hash table search?
 	for (sfx = known_sfx; sfx != NULL; sfx = sfx->next)
 		if(!strcmp (sfx->name, name))
 			return sfx;
@@ -925,6 +949,11 @@ sfx_t *S_PrecacheSound (const char *name, qboolean complain, qboolean lock)
 		return NULL;
 
 	sfx = S_FindName (name);
+
+	// clear the FILEMISSING flag so that S_LoadSound will try again on a
+	// previously missing file
+	sfx->flags &= ~ SFXFLAG_FILEMISSING;
+
 	if (sfx == NULL)
 		return NULL;
 
@@ -1258,6 +1287,7 @@ void S_StopSound(int entnum, int entchannel)
 		}
 }
 
+extern void CDAudio_Stop(void);
 void S_StopAllSounds (void)
 {
 	unsigned int i;
@@ -1266,6 +1296,9 @@ void S_StopAllSounds (void)
 	if (snd_renderbuffer == NULL)
 		return;
 
+	// stop CD audio because it may be using a faketrack
+	CDAudio_Stop();
+
 	for (i = 0; i < total_channels; i++)
 		S_StopChannel (i);
 
