@@ -37,6 +37,8 @@ cvar_t sv_protocolname = {0, "sv_protocolname", "DP7", "selects network protocol
 cvar_t sv_ratelimitlocalplayer = {0, "sv_ratelimitlocalplayer", "0", "whether to apply rate limiting to the local player in a listen server (only useful for testing)"};
 cvar_t sv_maxrate = {CVAR_SAVE | CVAR_NOTIFY, "sv_maxrate", "10000", "upper limit on client rate cvar, should reflect your network connection quality"};
 
+extern cvar_t sv_random_seed;
+
 static cvar_t sv_cullentities_pvs = {0, "sv_cullentities_pvs", "1", "fast but loose culling of hidden entities"}; // fast but loose
 static cvar_t sv_cullentities_trace = {0, "sv_cullentities_trace", "0", "somewhat slow but very tight culling of hidden entities, minimizes network traffic and makes wallhack cheats useless"}; // tends to get false negatives, uses a timeout to keep entities visible a short time after becoming hidden
 static cvar_t sv_cullentities_stats = {0, "sv_cullentities_stats", "0", "displays stats on network entities culled by various methods for each client"};
@@ -1234,7 +1236,7 @@ void SV_SendClientDatagram (client_t *client)
 	else
 	{
 		// PROTOCOL_DARKPLACES5 and later support packet size limiting of updates
-		maxrate = bound(NET_MINRATE, sv_maxrate.integer, NET_MAXRATE);
+		maxrate = max(NET_MINRATE, sv_maxrate.integer);
 		if (sv_maxrate.integer != maxrate)
 			Cvar_SetValueQuick(&sv_maxrate, maxrate);
 
@@ -1743,6 +1745,12 @@ void SV_SpawnServer (const char *server)
 	// level's data which is no longer valiud
 	cls.signon = 0;
 
+	if(*sv_random_seed.string)
+	{
+		srand(sv_random_seed.integer);
+		Con_Printf("NOTE: random seed is %d; use for debugging/benchmarking only!\nUnset sv_random_seed to get real random numbers again.\n", sv_random_seed.integer);
+	}
+
 	SV_VM_Setup();
 
 	sv.active = true;
@@ -1842,6 +1850,10 @@ void SV_SpawnServer (const char *server)
 	ent->fields.server->modelindex = 1;		// world model
 	ent->fields.server->solid = SOLID_BSP;
 	ent->fields.server->movetype = MOVETYPE_PUSH;
+	VectorCopy(sv.worldmodel->normalmins, ent->fields.server->mins);
+	VectorCopy(sv.worldmodel->normalmaxs, ent->fields.server->maxs);
+	VectorCopy(sv.worldmodel->normalmins, ent->fields.server->absmin);
+	VectorCopy(sv.worldmodel->normalmaxs, ent->fields.server->absmax);
 
 	if (coop.value)
 		prog->globals.server->coop = coop.integer;
